base_url = 'https://api.magic.link'

api_secret_api_key_missing_message = 'API secret key is missing. Please specify ' \
    'an API secret key when you instantiate the `Magic(api_secret_key=<KEY>)` ' \
    'object or use the environment variable, `MAGIC_API_SECRET_KEY`. You can ' \
    'get your API secret key from https://dashboard.magic.link. If you are having ' \
    'trouble, please don\'t hesitate to reach out to us at support@magic.link'
